#include "tid_config.h"

#if (TID_BUS_TYPE == TID_BUS_SPI)

#include "stm32f10x.h"
#include "bitband.h"
#include "port_spi.h"
#include "timer.h"


/********************************************************************
* Function: SPI SE interface and IO init
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void user_spi_init(void)
{
	//gpio init for SPI2
	//SPI_CS--PB12
	GPIOB->CRH &= 0xFFF0FFFF;
	GPIOB->CRH |= 0x00030000;
	PBout(12) = 1;

	//enable SPI2 clk
	RCC->APB1ENR |= 1<<14;
	//SPI_CLK--PB13,SPI_MISO--PB14,SPI_MOSI--PB15
	GPIOB->CRH &= 0x000FFFFF;
	GPIOB->CRH |= 0xBBB00000;

	//SPI2 init,36MHz/4=9MHz,master,CPOL=1,CPHA=1,
	SPI2->CR1 |= 3<<8 | 1<<6 | 1<<3 | 7;
}

/********************************************************************
* Function: SPI SE device power on
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void user_spi_dev_power_on(void)
{
	//SE_VCC_EN--PE4,
	GPIOE->CRL &= 0xFFF0FFFF;
	GPIOE->CRL |= 0x00030000;
	//power on
	PEout(4) = 1;
	HAL_DelayMs(SPI_PWR_ON_DELAY);
}

/********************************************************************
* Function: SPI SE device power off
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void user_spi_dev_power_off(void)
{
	//SE_VCC_EN--PE4,
	GPIOE->CRL &= 0xFFF0FFFF;
	GPIOE->CRL |= 0x00030000;
	//power off
	PEout(4) = 0;
	HAL_DelayMs(SPI_PWR_OFF_DELAY);
}

/********************************************************************
* Function: SPI SE device chip select
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void user_spi_chip_select(void)
{
	PBout(12) = 0;
	HAL_DelayUs(SPI_SEL_DELAY);
}

/********************************************************************
* Function: SPI SE device chip deselect
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void user_spi_chip_deselect(void)
{
	PBout(12) = 1;
	HAL_DelayUs(SPI_DESEL_DELAY);
}

/********************************************************************
* Function: Master transcieve a SPI Byte to slaver
* Input Parameter:
* ch: master send byte
* Output Parameter:None
* Return: master received byte from slaver
*********************************************************************/
uint8_t user_spi_transceive_char(uint8_t ch)
{
	uint8_t count;

	count = 100;
	//tx
	SPI2->DR = ch;
	do{
		count--;
	}while( !(SPI2->SR & 2) && count );

	count = 100;
	//rx
	do{
		count--;
	}while( !(SPI2->SR & 1) && count );

	ch = SPI2->DR;
	
	return ch;
}

//global variable of SPI interface driver
sTid_SPI_Driver gusr_spi_drv = 
{
	SE_SPI_NUM_1,
	user_spi_dev_power_on,
	user_spi_dev_power_off,
	user_spi_init,
	user_spi_chip_select,
	user_spi_chip_deselect,
	user_spi_transceive_char
};

#endif
