/**
******************************************************************************
* Copyright (c) 2019 Tencent. 
* All rights reserved.
******************************************************************************
* @file           : tid_sample.c
* @brief          : sample for how to use tid
******************************************************************************
*/
#include "tid_api_inf.h"
#include "tid_log.h"
#include "tid_app.h"

/*
static void _write_se_data()
{
    uint8_t test_data[256];
    uint16_t rlen;
    eTidResult ret;
    uint8_t tid_id_apdu[] = {
        0x80, 0xe2, 0x00, 0xa0, 0x13, 0x0b, 0x80, 0x10, 0x00, 0x00, 0x00, 0x00, 0xd2, 0x09, 0x6a, 0xc9, 0xb9, 0xf5, 0x1d, 0xf7, 0x68, 0x9b, 0x4c, 0x14
        };
        
    uint8_t tid_psk_apdu[] = {
        0x80, 0xe2, 0x00, 0x90, 0x23, 0x01, 0x41, 0x20, 0xc9, 0x9f, 0x1c, 0x23, 0x48, 0x8a, 0xc9, 0xfc, 0x67, 0x63, 0xc9, 0x94, 0xbe, 0x1e, 0x74, 0xb9, 0xe3, 0x51, 0x5b, 0xcc, 0xbf, 0x89, 0xaa, 0x8d, 0x16, 0x4a, 0x6c, 0xab, 0x01, 0x52, 0x37, 0x5d
        };
    uint8_t tid_priv_apdu[] = {
        0x80, 0xe2, 0x00, 0x80, 0x23, 0x0a, 0x74, 0x20, 0x17, 0xf4, 0xcc, 0x5e, 0x04, 0x4b, 0x9f, 0x8e, 0x17, 0xe7, 0x3e, 0xcd, 0x50, 0x82, 0x3f, 0xed, 0xed, 0x34, 0x2d, 0x80, 0x24, 0x3a, 0x61, 0xe2, 0x8a, 0x03, 0xbe, 0x7d, 0xd5, 0x7f, 0x2d, 0xb0
        };
    uint8_t tid_pub_apdu[] = {
        0x80, 0xe2, 0x00, 0x81, 0x45, 0x0a, 0x72, 0x20, 
        0xf0, 0xec, 0x4b, 0x2c, 0xe9, 0xfd, 0x91, 0xc0, 0x8d, 0x2b, 0x44, 0xdf, 0x2a, 0x42, 0x80, 0x53, 0x85, 0xdd, 0x44, 0x23, 0x50, 0x00, 0x7e, 0xcc, 0x1d, 0xb2, 0x6a, 0x4b, 0x11, 0xc9, 0xe0, 0x4b, 0x73, 0x20, 0x12, 0x37, 0x76, 0x87, 0x04, 0x59, 0x71, 0x9e, 0x99, 0x43, 0x29, 0xbc, 0x99, 0x41, 0xdf, 0x31, 0xa6, 0x3a, 0xd4, 0xc0, 0xe6, 0xdc, 0x92, 0xe9, 0xfc, 0x09, 0x8f, 0xe7, 0xb6, 0x28, 0x7a, 0xb8
        };
    ret = tid_raw_apdu(tid_id_apdu, sizeof(tid_id_apdu), test_data, &rlen);
    if(TID_ERR_SUCCESS != ret)
	{
		Log_e("Error %d", __LINE__);
        return;
	}

    ret = tid_raw_apdu(tid_psk_apdu, sizeof(tid_psk_apdu), test_data, &rlen);
    if(TID_ERR_SUCCESS != ret)
	{
		Log_e("Error %d", __LINE__);
        return;
	}

    ret = tid_raw_apdu(tid_priv_apdu, sizeof(tid_priv_apdu), test_data, &rlen);
    if(TID_ERR_SUCCESS != ret)
	{
		Log_e("Error %d", __LINE__);
        return;
	}

    ret = tid_raw_apdu(tid_pub_apdu, sizeof(tid_pub_apdu), test_data, &rlen);
    if(TID_ERR_SUCCESS != ret)
	{
		Log_e("Error %d", __LINE__);
        return;
	}    
}
*/

void tid_apdu_test(void)
{
    /* Init */
    int i, j;
    
	eTidResult ret;
    uint16_t rlen;
	sTidClient *ptidClient;
	eBusType type = eBUS_I2C;
	eVendor vendor = eVENDOR_FM;

    uint8_t test_data[512];
    
	ptidClient = (sTidClient  *)tid_client_init(type, vendor);	
	if (!ptidClient) {
        Log_e("Failed to init tid client");
        return;
    }

    ptidClient->pfun->tid_open_device();   //user_i2c_dev_power_on
	ptidClient->pfun->tid_device_init();   //user_i2c_init 

    ret = ptidClient->pfun->tid_dev_power_on(test_data, &rlen); //fm_i2c_get_atr --> fm_i2c_send_frame  --> tid_i2c_recv_frame 
	if(TID_ERR_SUCCESS != ret)
	{
		Log_e("tid_dev_power_on failed, Ret: %d", ret);
        goto TEST_EXIT;
	}
	else
	{
		Log_d("PowerOn:ret=%x,rlen=%d\r\n", ret, rlen);
	}

    /* Get Challenge Test */
    int test_lengths[] = {4, 8, 16};

    for (i = 0; i < sizeof(test_lengths) / sizeof(test_lengths[0]); ++i) {
        ret = tid_get_challenge(test_lengths[i], test_data, &rlen);
        if(TID_ERR_SUCCESS != ret)
        {
            Log_e("tid_get_challenge %d failed\r\n", test_lengths[i]);
        }
        else
        { 
            Log_d("tid_get_challenge %d OK\r\n", test_lengths[i]);
            HexDump("challenge bytes", test_data, rlen);
        }
    }

    /* Get Tid test */
    ret = tid_get_tid(test_data, &rlen);
    if(TID_ERR_SUCCESS != ret)
    {
        Log_e("tid_get_tid failed\r\n");
    }
    else
    { 
        Log_d("tid_get_tid OK\r\n");
        HexDump("tid data", test_data, rlen);
    }

    /* Get Device Public Key */
    ret = tid_get_dev_pubkey(test_data, &rlen);
    if(TID_ERR_SUCCESS != ret)
    {
        Log_e("tid get device pubkey failed\r\n");
    }
    else
    { 
        Log_d("tid get device pubkey OK\r\n");
        HexDump("device_pubkey", test_data, rlen);
    }    

    /* AES Cryption Test */
    do {
        for (i = 0; i < 32; ++i) {
            test_data[i] = 0x10 + i;
        }
        ret = tid_import_sesskey(test_data, 32, TID_SYM_KTYPE_AES_CBC, TID_KID_SYMKEY0);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid import session key failed");
            break;
        }

        /* single block AES256 */
        for (i = 0; i < 64; ++i) {
            test_data[i] = i;
        }
        ret = tid_sym_enc(TID_SYM_KTYPE_AES_CBC, test_data, 64, test_data, &rlen, TID_KID_SYMKEY0);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid sym encryption failed");
            break;
        }
        Log_d("AES256 CBC single block encryption succeeded");
        HexDump("AES encryption data", test_data, rlen);

        ret = tid_sym_dec(TID_SYM_KTYPE_AES_CBC, test_data, 64, test_data, &rlen, TID_KID_SYMKEY0);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid sym decryption failed");
            break;
        }
        Log_d("AES256 CBC single block decryption succeeded");
        HexDump("AES decryption data", test_data, rlen);

        /* multi block AES256 */
        for (i = 0; i < 512; ++i) {
            test_data[i] = i & 0x7f;
        }
        ret = tid_sym_enc(TID_SYM_KTYPE_AES_CBC, test_data, 512, test_data, &rlen, TID_KID_SYMKEY0);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid sym encryption failed");
            break;
        }
        Log_d("AES256 CBC multi block encryption succeeded");
        HexDump("AES encryption data", test_data, rlen);

        ret = tid_sym_dec(TID_SYM_KTYPE_AES_CBC, test_data, 512, test_data, &rlen, TID_KID_SYMKEY0);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid sym decryption failed");
            break;
        }
        Log_d("AES256 CBC multi block decryption succeeded");
        HexDump("AES decryption data", test_data, rlen);
    } while(0);

    /* TID ECC256 SECP256R1 Sign and Inner Verify Test */
    do {
        uint8_t _signature[64];
        for (i = 0; i < 138; ++i) {
            test_data[i] = i;
        }
    
        ret = tid_asym_crypto_sign(test_data, 138, _signature, &rlen, TID_KID_PRIVKEY, TID_ASYM_ALGO_ECSDA);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid sym sign failed");
            break;
        }
        Log_d("ECSDA Sign succeeded");
        HexDump("Signature", _signature, rlen);

        ret = tid_asym_crypto_verify_inner(test_data, 138, _signature, TID_KID_PUBKEY, TID_ASYM_ALGO_ECSDA);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid sym verify inner failed");
            break;
        }
        Log_d("ECSDA Verify inner succeeded");

        test_data[0] += 1;          /* Change the test_data and verify again */
        ret = tid_asym_crypto_verify_inner(test_data, 138, _signature, TID_KID_PUBKEY, TID_ASYM_ALGO_ECSDA);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid sym verify inner failed");
            break;
        }
        Log_d("ECSDA Verify inner succeeded");        
    } while(0);

    /* TID ECC256 SECP256R1 Verify Outer Test */
    do {
        uint8_t _sign_msg[] = {
            0x24, 0xc8, 0xc4, 0x26, 0x97, 0x14, 0xf0, 0xcf, 0xdc, 0xfc, 0x35, 0x90, 0x00, 0x29, 0xb1, 0x08, 
            0xdb, 0x19, 0x47, 0xf7, 0xf4, 0x11, 0x98, 0xcc, 0xe6, 0xd2, 0xc7, 0x05, 0x5a, 0x71, 0xb7, 0x26
            };
        uint8_t _signature[] = {
            0x94, 0x81, 0x85, 0x2c, 0xd7, 0x47, 0xd9, 0xdd, 0x50, 0x7f, 0x03, 0x83, 0xf2, 0x73, 0x30, 0xf4, 
            0x2a, 0xa3, 0x2d, 0xfb, 0x9b, 0x44, 0x26, 0x27, 0x54, 0x05, 0x4d, 0x53, 0x9e, 0x20, 0xde, 0x93,
            0x86, 0xea, 0x81, 0x69, 0xcd, 0x2c, 0xf7, 0x78, 0x74, 0xa9, 0xc8, 0x18, 0x2b, 0xaa, 0x84, 0xa1, 
            0x81, 0xc9, 0x61, 0x50, 0xb0, 0x64, 0x31, 0xaa, 0x4f, 0x98, 0x04, 0x09, 0x61, 0x5c, 0xe4, 0x1c
            };
        uint8_t _pubkey[] = {
            0xf0, 0xec, 0x4b, 0x2c, 0xe9, 0xfd, 0x91, 0xc0, 0x8d, 0x2b, 0x44, 0xdf, 0x2a, 0x42, 0x80, 0x53, 
            0x85, 0xdd, 0x44, 0x23, 0x50, 0x00, 0x7e, 0xcc, 0x1d, 0xb2, 0x6a, 0x4b, 0x11, 0xc9, 0xe0, 0x4b, 
            0x12, 0x37, 0x76, 0x87, 0x04, 0x59, 0x71, 0x9e, 0x99, 0x43, 0x29, 0xbc, 0x99, 0x41, 0xdf, 0x31, 
            0xa6, 0x3a, 0xd4, 0xc0, 0xe6, 0xdc, 0x92, 0xe9, 0xfc, 0x09, 0x8f, 0xe7, 0xb6, 0x28, 0x7a, 0xb8
            };
        ret = tid_asym_crypto_verify_outer(_sign_msg, sizeof(_sign_msg), _signature, _pubkey, TID_ASYM_ALGO_ECSDA);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid sym verify outer failed");
            break;
        }
        Log_d("ECSDA Verify outer succeeded");

        _sign_msg[0] += 1;          // Change the sign message and test again
        ret = tid_asym_crypto_verify_outer(_sign_msg, sizeof(_sign_msg), _signature, _pubkey, TID_ASYM_ALGO_ECSDA);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid sym verify outer failed");
            break;
        }
        Log_d("ECSDA Verify outer succeeded");
    } while(0);

    /* Key exchange Test */
    do {
        uint8_t _that_pubkey[] = {
            0x24, 0xbd, 0x72, 0x8f, 0x74, 0xe8, 0x4a, 0x72, 0xdd, 0xaa, 0xa9, 0x08, 0x1b, 0x91, 0x21, 0x8f, 
            0x6c, 0xed, 0x84, 0xdd, 0xfb, 0x87, 0x1e, 0x2a, 0xda, 0x4b, 0x6a, 0x68, 0x1f, 0x39, 0xd3, 0xb9, 
            0xc2, 0x26, 0x2b, 0x18, 0x90, 0xa3, 0x5d, 0x59, 0x89, 0x9d, 0x95, 0x6d, 0x3f, 0x1a, 0x23, 0x77, 
            0xdd, 0x52, 0xfe, 0x2d, 0xc0, 0x6f, 0xb2, 0x34, 0xc1, 0x9e, 0x7e, 0x95, 0x7e, 0x8e, 0x0d, 0xb6
            };
        ret = tid_gen_temp_keypair(test_data, &rlen, TID_KTYPE_ECC256);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid generate temporary key pair failed");
            break;
        }
        Log_d("Generate temporary key pair succeeded");
        HexDump("Temporary public key", test_data, rlen);

        ret = tid_gen_token(_that_pubkey, sizeof(_that_pubkey), TID_ECDH_ALGO_ECC256, 0, test_data);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid generate shared key pair failed");
            break;
        }
        Log_d("Generate shared key succeeded");
        HexDump("Temporary Shared key", test_data, 64);
   }while(0);

    /* Test ECDH with internal shared key */
    do {
        uint8_t _that_pubkey[] = {
            0x24, 0xbd, 0x72, 0x8f, 0x74, 0xe8, 0x4a, 0x72, 0xdd, 0xaa, 0xa9, 0x08, 0x1b, 0x91, 0x21, 0x8f, 
            0x6c, 0xed, 0x84, 0xdd, 0xfb, 0x87, 0x1e, 0x2a, 0xda, 0x4b, 0x6a, 0x68, 0x1f, 0x39, 0xd3, 0xb9, 
            0xc2, 0x26, 0x2b, 0x18, 0x90, 0xa3, 0x5d, 0x59, 0x89, 0x9d, 0x95, 0x6d, 0x3f, 0x1a, 0x23, 0x77, 
            0xdd, 0x52, 0xfe, 0x2d, 0xc0, 0x6f, 0xb2, 0x34, 0xc1, 0x9e, 0x7e, 0x95, 0x7e, 0x8e, 0x0d, 0xb6
            };
        uint8_t _prf[] = {0x01, 0x02, 0x03, 0x04};
        ret = tid_gen_temp_keypair(test_data, &rlen, TID_KTYPE_ECC256);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid generate temporary key pair failed");
            break;
        }
        Log_d("Generate temporary key pair succeeded");
        HexDump("Temporary public key", test_data, rlen);

        ret = tid_gen_token(_that_pubkey, sizeof(_that_pubkey), TID_ECDH_ALGO_ECC256, 1, NULL);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid generate shared key failed");
            break;
        }
        Log_d("Generate shared key succeeded");

        ret = tid_gen_session_key(_prf, sizeof(_prf));
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid generate inernal session key failed");
            break;
        }
        Log_d("Generate session key succeeded");

        for (i = 0; i < 64; ++i) {
            test_data[i] = i;
        }
        ret = tid_sym_enc(TID_SYM_KTYPE_AES_CBC, test_data, 64, test_data, &rlen, TID_KID_SESSKEY);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid internal session key encryption failed");
            break;
        }
        Log_d("internal session key encryption succeeded");
        HexDump("Internal encryption result", test_data, rlen);
   }while(0);

    /* TID Compute Digest Test */
   int digest_kids[] = {-1, 0x90};  /* KID of -1 means only input data, KID of 0x90 means Dev-PSK || input data */
   for (j = 0; j < 2; ++j) {
        /* Single block SHA256 Test */
        for (i = 0; i < 64; ++i) {
            test_data[i] = i;
        }

        ret = tid_compute_digest(digest_kids[j], test_data, 64, test_data, &rlen, TID_DIGEST_SHA256);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid single block digest computation kid %d failed", digest_kids[j]);
            break;
        }
        Log_d("tid single block digest computation kid %d succeeded", digest_kids[j]);
        HexDump("Sha256 digest", test_data, rlen);

        /* Multi block SHA256 Test */
        for (i = 0; i < 512; ++i) {
            test_data[i] = i & 0x7f;
        }

        ret = tid_compute_digest(digest_kids[j], test_data, 512, test_data, &rlen, TID_DIGEST_SHA256);
        if (TID_ERR_SUCCESS != ret) {
            Log_e("tid multi block digest computation kid %d failed", digest_kids[j]);
            break;
        }
        Log_d("tid multi block digest computation kid %d succeeded", digest_kids[j]);
        HexDump("Sha256 digest", test_data, rlen);
    }

TEST_EXIT:
    ptidClient->pfun->tid_close_device();
}


