/**
  ******************************************************************************
  * Copyright (c) 2019 Tencent. 
  * All rights reserved.
  ******************************************************************************

  ******************************************************************************
  * @file           : tid_driver_common.c
  * @brief          : tid driver wrapper for kinds of se device vendor
  ******************************************************************************
*/
#include "tid_driver_comm.h"
#include "fm_driver_i2c.h"
#include "port_i2c.h"
//#include "gpio_i2c.h"
#include "fm_driver_spi.h"
#include "port_spi.h"


static sDrvierInfo g_DriverTable[] = 
{
	{eVENDOR_FM, eBUS_I2C, &gfm_func_i2c},
//	{eVENDOR_FM, eBUS_SPI, &gfm_func_spi},
	{eVENDOR_FM, eBUS_UART, NULL},
	{eVENDOR_FM, eBUS_7816, NULL},

	{eVENDOR_HDE, eBUS_I2C, NULL},
	{eVENDOR_HDE, eBUS_SPI, NULL},
	{eVENDOR_HDE, eBUS_UART, NULL},
	{eVENDOR_HDE, eBUS_7816, NULL},

	{eVENDOR_NXP, eBUS_I2C, NULL},
	{eVENDOR_NXP, eBUS_SPI, NULL},
	{eVENDOR_NXP, eBUS_UART, NULL},
	{eVENDOR_NXP, eBUS_7816, NULL},

	{eVENDOR_WY, eBUS_I2C, NULL},
	{eVENDOR_WY, eBUS_SPI, NULL},
	{eVENDOR_WY, eBUS_UART, NULL},
	{eVENDOR_WY, eBUS_7816, NULL},

	{eVENDOR_COMMON, eBUS_I2C, NULL},
	{eVENDOR_COMMON, eBUS_SPI, NULL},
	{eVENDOR_COMMON, eBUS_UART, NULL},
	{eVENDOR_COMMON, eBUS_7816, NULL}
};


static sTidFunc *_get_tid_func_handle(eBusType type, eVendor vendor)
{
	sTidFunc *pfun = NULL;
	
	for(int i=0; i < sizeof(g_DriverTable)/sizeof(g_DriverTable[0]); i++)
	{
		if(vendor == g_DriverTable[i].vendor)
		{
			if(type == g_DriverTable[i].type)
			{
				pfun = 	g_DriverTable[i].pfun;
				break;
			}
		}
		else
		{
			continue;
		}
	}

	return pfun;	
}

sTidFunc *tid_se_register(eBusType type, eVendor vendor)
{
	sTidFunc *pgtid_SeFunc;

	pgtid_SeFunc = _get_tid_func_handle(type, vendor);
	if(NULL == pgtid_SeFunc)
	{
		Log_e("Vendor(%d)-BusType(%d) not support,pls check or add new driver", vendor, type);
		goto end;
	}
	
	if(eBUS_I2C == type)
	{
		pgtid_SeFunc->tid_driver_register(&gusr_i2c_drv);
		//pgtid_SeFunc->tid_driver_register(&gusr_gpio_i2c_drv);
	}
	else if(eBUS_SPI == type)
	{
		//pgtid_SeFunc->tid_driver_register(&gusr_spi_drv);
	}
	else if(eBUS_UART == type)
	{
		//pgtid_SeFunc->tid_driver_register(&gusr_uart_drv);
		Log_e("Bus type not supported yet");
		pgtid_SeFunc = NULL;
	}
	else if(eBUS_7816 == type)
	{
		//pgtid_SeFunc->tid_driver_register(&gusr_7816_drv);
		Log_e("Bus type not supported yet");
		pgtid_SeFunc = NULL;
	}
	else
	{
		Log_e("Bus type not supported yet");
		pgtid_SeFunc = NULL;
	}

end:		
	return pgtid_SeFunc;
}

void tid_se_unregister(sTidFunc *pfun)
{
	if(NULL != pfun)
	{
		pfun->tid_driver_unregister();
	}
		
	pfun = NULL;
}
