/*
 * Tencent is pleased to support the open source community by making IoT Hub available.
 * Copyright (C) 2016 THL A29 Limited, a Tencent company. All rights reserved.

 * Licensed under the MIT License (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://opensource.org/licenses/MIT

 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifdef __cplusplus
extern "C" {
#endif



#include <string.h>
#include "tid_log.h"


#define MAX_LOG_MSG_LEN 			(255)

#if defined(__linux__)
	#undef  MAX_LOG_MSG_LEN
	#define MAX_LOG_MSG_LEN                  (1023)
#endif

static char *level_str[] = {
    "DBG", "INF", "WRN", "ERR",
};

LOG_LEVEL g_log_level = LOG_DEBUG;

extern void HAL_Printf(const char *fmt, ...);


static const char *_get_filename(const char *p)
{
    char ch = '/';
    const char *q = strrchr(p,ch);
    if(q == NULL)
    {
        q = p;
    }
    else
    {
        q++;
    }
    return q;
}

void Log_writter(const char *file, const char *func, const int line, const int level, const char *fmt, ...)
{
	if (level < g_log_level) {
		return;
	}

	const char *file_name = _get_filename(file);

    HAL_Printf("%s|%s|%s(%d): ", level_str[level], file_name, func, line);

    va_list ap;
    va_start(ap, fmt);
    vprintf(fmt, ap);
    va_end(ap);

    HAL_Printf("\r\n");

    return;
}

void HexDump(const char *bufName, const unsigned char *pData, unsigned int len)
{
    if (bufName) {
        HAL_Printf("%s:\r\n", bufName);
    }
	for(int i = 0; i < len; i++)
	{
		HAL_Printf("%02X ",pData[i]);
	}
	HAL_Printf("\r\n");
}


#ifdef __cplusplus
}
#endif
