#include "tid_export.h"
#if ((TID_SE_VENDOR == TID_SE_VENDOR_WY) && (TID_BUS_TYPE == TID_BUS_I2C))


#include <stdint.h>
#include "port_i2c.h"
#include "tid_log.h"
#include "tid_delay.h"
#include "wy_driver_i2c.h"
//variable of I2C SE addr

static uint8_t gwy_I2CAddr;

//struct point of tid I2C driver
static sTid_I2C_Driver *pgwy_I2CDrv = NULL;
uint8_t i2c_send_buf[1024]={0};
uint16_t i2c_send_len=0;

static eTidResult wy_i2c_transfer_result(uint8_t ret)
{
	eTidResult resault;
	switch(ret)
	{
		case 0:
			resault = TID_ERR_SUCCESS;
			break;
	
		case 1:
			resault = TID_ERR_CRC;
			break;
		case 2:
			resault = TID_ERR_INS;
			break;
		case 11:
			resault = TID_ERR_NULL;
			break;
		case 12:
			resault = TID_ERR_RECV_ACK;
			break;
		case 13:
			resault = TID_ERR_LENGTH;
			break;
		case 14:
			resault = TID_ERR_LRC;
			break;
		default:
			resault = TID_ERR_OTHER;
			break;
	}

	return resault;
}


/********************************************************************
* Function: register I2C driver
* Input Parameter:
* wy_i2c_drv: the struct point of I2C driver
* Output Parameter:None
* Return: None
*********************************************************************/
void wy_i2c_drv_reg(sTid_I2C_Driver *wy_i2c_drv)
{
	pgwy_I2CDrv = wy_i2c_drv;
}

/********************************************************************
* Function: unregister I2C driver
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void wy_i2c_drv_unreg(void)
{
	pgwy_I2CDrv = NULL;
}

/********************************************************************
* Function: I2C device init
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void wy_i2c_dev_init(void)
{
	if(pgwy_I2CDrv)
		pgwy_I2CDrv->tid_i2c_init();
}

/********************************************************************
* Function: open I2C device
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void wy_i2c_open_device(void)
{
	log_debug("wy_i2c_open_device\r\n");
	if(pgwy_I2CDrv)
	{
		gwy_I2CAddr = pgwy_I2CDrv->se_i2c_addr;
		pgwy_I2CDrv->tid_i2c_power_on();
	}
}

/********************************************************************
* Function: close I2C device
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void wy_i2c_close_device(void)
{
	log_debug("wy_i2c_close_device\r\n");
	if(pgwy_I2CDrv)
	{
		gwy_I2CAddr = 0;
		pgwy_I2CDrv->tid_i2c_power_off();
	}
}

/********************************************************************
* Function: send a frame to I2C SE
* Input Parameter:
* wy_i2c_drv: the struct point of I2C driver
* sbuf: send frame data
* slen: send frame length
* Output Parameter:None
* Return: result 
* 0-success
* 11-null point err
* 12-wait ack timeout
* other-RFU
*********************************************************************/
eTidResult wy_i2c_send_frame(uint8_t cmd, uint8_t *sbuf, uint16_t slen )
{
	WY_I2C_HEAD wy_i2c_hd;
	uint16_t i;
	uint8_t bcc;
	eTidResult ret;
	WY_TxBuf Wy_TxBuf;
	Wy_TxBuf.lenlo = slen+3;
	Wy_TxBuf.lenhi = (slen+3)>>8;
	Wy_TxBuf.nad = 0;
	Wy_TxBuf.cmd = cmd;
	bcc = Wy_TxBuf.lenlo ^ Wy_TxBuf.lenhi ^ Wy_TxBuf.nad ^ Wy_TxBuf.cmd;
    for(i=0; i<slen; i++)
	{
		bcc ^= sbuf[i];
		Wy_TxBuf.buf[i]=sbuf[i];		
	}
    Wy_TxBuf.buf[slen]=bcc;
	//log_array_u8((uint8_t *)&Wy_TxBuf,slen+5,"Wy_TxBuf");
	ret =pgwy_I2CDrv->tid_i2c_send_buf((uint8_t *)&Wy_TxBuf,slen+5);
	if(TID_ERR_SUCCESS != ret) 
	{
		log_debug("send txbuf err\r\n");
	}
END:
	return ret;
}



/********************************************************************
* Function: receive a frame from I2C SE
* Input Parameter:None
* Output Parameter:
* rbuf: received frame data
* rlen: received frame length
* Return: result 
* 0-success
* 1-SE CRC err
* 2-SE INS err
* 0xF2-SE WTX err
* 11-null point err
* 12-receive ack err
* 13-received length err
* 14-received bcc err
* other-RFU
*********************************************************************/
eTidResult wy_i2c_recv_frame(uint8_t *rbuf, uint16_t *rlen)
{
	uint16_t i;
	uint16_t recvLen;
	uint16_t len;
	uint8_t bcc;
	eTidResult ret;	
	WY_RxBuf Wy_RxBuf;
	uint16_t rxlen_exp=0;
	rxlen_exp=*rlen+5; 										//rxlen_exp=*rlen(data+sw0+sw1) + 5(LenHLenLNADSTACHECKSUM)
	ret = pgwy_I2CDrv->tid_i2c_recv_buf((uint8_t *)&Wy_RxBuf,rxlen_exp);
	recvLen  = (Wy_RxBuf.lenhi << 8) + Wy_RxBuf.lenlo; 		//chack length

	if((recvLen < I2C_MIN_LEN) || (recvLen > I2C_MAX_LEN) ||(recvLen!=rxlen_exp-2))   //recvLen= rxlen_exp-2(LenHLenL)
	{
		if(recvLen==I2C_MIN_LEN)  			//NAD+STA+CHECKSUM(NAK:SEָУCRC)
		{
			*rlen=0; 
			goto check_rev;
		}
		else if(recvLen==I2C_MIN_LEN+2) 	//NAD+STA+SW0+SW1+CHECKSUM (APDUȨ޲Ҫ)
		{
			*rlen=2; 						//SW0+SW1
			goto check_rev;
		}
		else 								//receive err len
		{
			log_debug("err recvLen=%04x\r\n",recvLen);
			log_array_u8((uint8_t *)&Wy_RxBuf,rxlen_exp,"Wy_RxBuf");
		}
		return TID_ERR_LENGTH;
	}
	//recv nad and sta
check_rev:
	bcc = Wy_RxBuf.lenlo ^ Wy_RxBuf.lenhi ^ Wy_RxBuf.nad ^ Wy_RxBuf.sta;	
	//recv data
	for(i=0; i<*rlen; i++)
	{
		rbuf[i] = Wy_RxBuf.buf[i];
		bcc ^= rbuf[i];
	}
	bcc ^= Wy_RxBuf.buf[*rlen];
	if(bcc)
	{
		return TID_ERR_LRC;
	}
	if(0x0 ==Wy_RxBuf.sta)
	{
		ret = TID_ERR_SUCCESS;
	}
	else if(0x1 == Wy_RxBuf.sta)
	{
		ret = TID_ERR_CRC;
	}
	else if(0xF2 ==Wy_RxBuf.sta)
	{
		ret = TID_ERR_DATA_NOT_READY;
	}
	else
	{
		ret = TID_ERR_FAILURE;
	}
	//log_debug("recv stat:%02x\r\n",Wy_RxBuf.sta);
	return ret;
}
/********************************************************************
* Function: get ATR from I2C SE
* Input Parameter:None
* Output Parameter:
* rbuf: received ATR data
* rlen: received ATR length
* Return: result 
* 0-success
* 1-SE CRC err
* 2-SE INS err
* 0xF2-SE WTX err
* 11-null point err
* 12-receive ack err
* 13-received length err
* 14-received bcc err
* other-RFU
*********************************************************************/
eTidResult wy_i2c_get_atr(uint8_t *rbuf, uint16_t *rlen)
{
	eTidResult ret;
	uint8_t retry_cnt = I2C_RETRY_TIMES;
	uint8_t request_cnt = I2C_REQUEST_TIMES;
	TID_FUNC_ENTRY
RETRY:
    log_debug("wy_i2c_get_atr\r\n");
	ret = wy_i2c_send_frame(I2C_CMD_GET_ATR, 0, 0);
	if(TID_ERR_SUCCESS != ret) 
	{
		log_debug("Error:wy_i2c_send_frame err,%d",ret);
		return ret;
	}	
	tid_delay_ms(POLL_TIMEOUT);
	*rlen=3;
	do{
		ret = wy_i2c_recv_frame(rbuf, rlen);
		if(ret == TID_ERR_RECV_ACK || ret == TID_ERR_DATA_NOT_READY)
		{	//timeout
			tid_delay_ms(POLL_TIMEOUT);
		}
		else if(ret == TID_ERR_LENGTH || ret == TID_ERR_LRC)
		{	//receive err
			if(!request_cnt)
			{
				break;
			}				
			request_cnt--;
			wy_i2c_send_frame(I2C_CMD_NAK, 0, 0);	//request SE resend
		}
		else
		{
			break;
		}
	}while(1);
	if(ret == TID_ERR_CRC && retry_cnt)	//SE CRC err
	{
		retry_cnt--;
		goto RETRY;
	}
	//log_debug("wy_i2c_get_atr ret=%x\r\n", ret);
	return ret;
}

/********************************************************************
* Function: transcieve a frame to I2C SE
* Input Parameter:
* sbuf: send frame data
* slen: send frame length
* timeout: timeout for wait receive a frame
* Output Parameter:
* rbuf: received frame data
* rlen: received frame length
* Return: result 
* 0-success
* 1-SE CRC err
* 2-SE INS err
* 0xF2-SE WTX err
* 11-null point err
* 12-receive ack err
* 13-received length err
* 14-received bcc err
* other-RFU
*********************************************************************/
eTidResult wy_i2c_transceive(uint8_t *sbuf, uint16_t slen, uint8_t *rbuf, uint16_t *rlen, 
						uint16_t poll_inv, uint32_t poll_timeout)
{	
	uint8_t retry_cnt = I2C_RETRY_TIMES;
	uint8_t request_cnt = I2C_REQUEST_TIMES;
	eTidResult ret;
RETRY:
#ifdef WY_DEBUG_EABLE
	log_debug("wy_i2c_transceive data dump:");
	HexDump(sbuf, slen);
#endif
	
    log_array_u8(sbuf,slen,"apdu");
	
	ret = wy_i2c_send_frame(I2C_CMD_IBLOCK, sbuf, slen);

	//log_debug("wy_i2c_send_frame ret=%02x\r\n",ret);

	if(TID_ERR_SUCCESS != ret)
	{
		log_debug("wy_i2c_send_frame err,%d", ret);
		return TID_ERR_FAILURE;
	}	
	//HAL_Timer_countdown_ms(&timer, poll_timeout);
	tid_delay_ms(poll_timeout);
	do{
		ret = wy_i2c_recv_frame(rbuf, rlen);
		//log_array_u8(rbuf,*rlen,"rbuf");
		if(ret == TID_ERR_RECV_ACK || ret == TID_ERR_DATA_NOT_READY)
		{	//timeout
			tid_delay_ms(poll_inv);
		}
		else if(ret == TID_ERR_LENGTH || ret == TID_ERR_LRC)
		{	//receive err
			if(!request_cnt)
			{
				break;
			}				
			request_cnt--;
			wy_i2c_send_frame(I2C_CMD_NAK, 0, 0);	//request SE resend
		}
		else
		{
			break;
		}
			
	}while(1);
		
	//log_debug("recv_ret=%x\r\n",ret);
	
	if(ret == TID_ERR_CRC && retry_cnt)	//SE CRC err
	{
		retry_cnt--;
		goto RETRY;
	}

	return	ret;
}


/********************************************************************
* Function: get ATR from I2C SE
* Input Parameter:None
* Output Parameter:
* rbuf: received ATR data
* rlen: received ATR length
* Return: result 
* 0-success
* 1-SE CRC err
* 2-SE INS err
* 0xF2-SE WTX err
* 11-null point err
* 12-receive ack err
* 13-received length err
* 14-received bcc err
* other-RFU
*********************************************************************/
eTidResult wy_i2c_standby(uint8_t *rbuf, uint16_t *rlen)
{
	eTidResult ret;
	TID_FUNC_ENTRY
	uint8_t retry_cnt = I2C_RETRY_TIMES;
	uint8_t request_cnt = I2C_REQUEST_TIMES;
	
RETRY:	
    log_debug("wy_i2c_standby\r\n");
	ret = wy_i2c_send_frame(I2C_CMD_STANDBY, 0, 0);
	if(TID_ERR_SUCCESS != ret) 
	{
		log_debug("Error:wy_i2c_send_frame err,%d",ret);
		return ret;
	}	
	tid_delay_ms(POLL_TIMEOUT);
	*rlen=3;
	do{
		ret = wy_i2c_recv_frame(rbuf, rlen);
		if(ret == TID_ERR_RECV_ACK || ret == TID_ERR_DATA_NOT_READY)
		{	//timeout
			tid_delay_ms(POLL_TIMEOUT);
		}
		else if(ret == TID_ERR_LENGTH || ret == TID_ERR_LRC)
		{	//receive err
			return ret;
		}
		else
		{
			break;
		}
	}while(1);
	
	if(ret == TID_ERR_CRC && retry_cnt)	//SE CRC err
	{
		retry_cnt--;
		goto RETRY;
	}
	return ret;
}
/********************************************************************
* Function: open I2C device
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void wy_i2c_wake(void)
{
	if(pgwy_I2CDrv)
	{
		gwy_I2CAddr = pgwy_I2CDrv->se_i2c_addr;
		pgwy_I2CDrv->tid_i2c_wake();
	}
}


//global variable of I2C SE 
sTidFunc gwy_func_i2c = 
{
	eBUS_I2C,
	wy_i2c_drv_reg,
	wy_i2c_dev_init,
	wy_i2c_open_device,
	wy_i2c_close_device,
	wy_i2c_get_atr,
	wy_i2c_transceive,
	wy_i2c_drv_unreg,
	wy_i2c_standby,
	wy_i2c_wake
};

#endif
