import os
import binascii
import hashlib
import base64
import random
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend
#from Tools.scripts.pysource import binary_re
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.kdf.hkdf import HKDF
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import utils
from cryptography.exceptions import InvalidSignature

if __name__ == '__main__':
    if __package__ is None:
        import sys
        from os import path
        sys.path.append('./gmssl')

from gmssl.sm4 import CryptSM4, SM4_ENCRYPT, SM4_DECRYPT
from gmssl import sm3, func
from gmssl.sm2 import CryptSM2

testTidStr = "021000005D147EE031E5A618A3E0FED0"
testPskStr = "7b112e25ad199a4dfed1386e4da8d7398db6671ae0b984c3c0fd90203d5cc0ad"
devPriv = ""
devPub = ""

nonceStr = ""
mySignInfoStr = "94290c7105d8e4dcfa0c4dfe10228101fd40f4bdcb926e57bd525bf83ae670c6"

sm2PrivKey = "94290c7105d8e4dcfa0c4dfe10228101fd40f4bdcb926e57bd525bf83ae670c6"
sm2Key = CryptSM2(private_key=sm2PrivKey)

class SuiteConst(object):
    def __init__(self):
        self.FDM_TEST = 1
        self.SM2_SM4 = 16
        self.SM2_AES128 = 17
        self.SM2_AES256 = 18
        self.ECC256_SM4 = 19
        self.ECC256_AES128 = 20
        self.ECC256_AES256 = 21

suiteConst = SuiteConst()

def getKeyPairEcc():
    f = open('privFile.txt', 'r')
    bs = f.read(1024)
    f.close()
    myPrivKey = serialization.load_der_private_key(binascii.unhexlify(bs),
            password=b'1234', backend=default_backend())
    myPubKey = myPrivKey.public_key()

    return (myPrivKey, myPubKey)


def getAuthCodeEcc(tidStr, pskStr):
    myPrivKey, myPubKey = getKeyPairEcc()

    protVer = "01"
    suiteId = tidStr[2:4]

    randArr = []
    for i in range(16):
        randArr.append(random.randint(0, 255))
    randBytes = bytes(randArr)
    randStr = randBytes.hex()
    nonceStr = randStr

    signBuf = protVer + suiteId + tidStr + randStr + "0000" + hex(myPubKey.public_numbers().x)[2:] + hex(myPubKey.public_numbers().y)[2:]
    signInfo = hashlib.sha256(binascii.unhexlify(pskStr + signBuf)).hexdigest()
    authcode = binascii.unhexlify(signBuf + signInfo)

    return authcode

def serverAuthcodeVerifyEcc(authcode, tidStr, pskStr):
    thatSign = authcode[84:]
    signBuf = pskStr + tidStr + mySignInfoStr + authcode[2:2 + 16 + 66].hex()
    print("signBuf", signBuf)
    print(binascii.unhexlify(signBuf).hex())
    calcSign = hashlib.sha256(binascii.unhexlify(signBuf)).hexdigest()
    print(thatSign.hex(), calcSign)

def getKeyPairSm2(privKey=sm2PrivKey):
    sm2Key = CryptSM2(private_key=privKey)

    return (binascii.unhexlify(sm2PrivKey), binascii.unhexlify(sm2Key.get_pubkey()))

def getAuthCodeSm2(tidStr, pskStr, isSm):
    myPrivKey, myPubKey = getKeyPairSm2()

    protVer = "01"
    sId = int(tidStr[2:4], 16)
    if isSm == 1:
        sId = sId | 0x80
    suiteId = hex(sId)[2:]

    randArr = []
    for i in range(16):
        randArr.append(random.randint(0, 255))
    randBytes = bytes(randArr)
    randStr = randBytes.hex()
    nonceStr = randStr

    signBuf = protVer + suiteId + tidStr + randStr + "0000" + myPubKey.hex()
    if isSm == 1:
        signInfo = sm3.sm3_hash(func.bytes_to_list(binascii.unhexlify(pskStr + signBuf)))
    else:
        signInfo = hashlib.sha256(binascii.unhexlify(pskStr + signBuf)).hexdigest()
    authcode = binascii.unhexlify(signBuf + signInfo)

    return authcode

def serverAuthcodeVerifySm2(tidStr, pskStr, authcodeStr):
    thatSign = authcode[84:]
    signBuf = pskStr + tidStr + mySignInfoStr + authcode[2:2 + 16 + 66].hex()
    print("signBuf", signBuf)
    print(binascii.unhexlify(signBuf).hex())
    calcSign = hashlib.sha256(binascii.unhexlify(signBuf)).hexdigest()
    print(thatSign.hex(), calcSign)

def getRandBytes(len):
    randArr = []
    for i in range(len):
        randArr.append(random.randint(0, 255))
    return bytes(randArr)

def getAuthCode(tidStr, pskStr):
    suiteNum = int(tidStr[2:4], 16)

    if suiteNum == suiteConst.SM2_AES128 or suiteNum == suiteConst.SM2_AES256 or suiteNum == suiteConst.SM2_SM4:
        return getAuthCodeSm2(tidStr, pskStr, 1)
    else:
        return getAuthCodeEcc(tidStr, pskStr)

if (len(sys.argv) == 1):
    authcode = getAuthCode(testTidStr, testPskStr)
elif (len(sys.argv) == 3):
    authcode = getAuthCode(sys.argv[1], sys.argv[2])
else:
    print("Usage: python tid_sim.py tid psk\nThe tid and psk are HEX format!")
print("The authcode is ")
print(str(base64.b64encode(authcode), encoding="utf-8"))
